/*  "Expressive" compiler
 *
 *  Global fields and methods of the compiler.
 *  Philippe Altherr
 *
 *  $Id: Global.java,v 1.3 2003/03/21 17:23:16 mihaylov Exp $
 */

package expressive;

import java.io.InputStream;
import java.io.FileInputStream;
import java.io.OutputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.FileNotFoundException;

/**
 * This class defines methods for printing error messages and
 * debugging information as well as manipulating files
 */
public class Global {

    //########################################################################
    // Instance variables

    /** The unique Global instance. */
    public static Global instance;

    /** The name of the source file. */
    public final String source;

    /** Indicates whether to print the debugging messages. */
    public final boolean debug;

    /** Error counter. */
    private int errors;

    /** Exception used by the fatal(...) methods. */
    private final Error abort;

    //########################################################################
    // Constructors

    public Global(AbstractMain main) {
        if (instance != null) throw new Error("Second instance of Global");
        instance = this;
        this.source = main.source;
        this.debug = main.debug;
        this.abort = main.abort;
        this.errors = 0;
    }

    //########################################################################
    // Methods - error and debugging messages

    /** Returns the total number of errors. */
    public int errors() {
        return errors;
    }

    /** Print the position and the error message. */
    public void error(int position, String message) {
        errors++;
        print(position, message);
    }

    /** Print the position and the message and abort the compiler. */
    public Error fatal(int position, String message) {
        error(position, message);
        throw abort;
    }

    /** Print the position and the exception and abort the compiler. */
    public Error fatal(int position, Exception exception) {
        String message = exception.getMessage();
        if (message == null)
            message = "Fatal error(" + exception.getClass() + ")";
        return fatal(position, message);
    }

    /** Print the position and the message if debugging is enabled. */
    public void debug(int position, String message) {
        if (debug) print(position, "[debug] " + message);
    }

    /** Print the position and the message. */
    private void print(int position, String message) {
        if (source != null && !source.equals("-")) {
            System.err.print(source);
            System.err.print(":");
        }
	if (position != Position.UNDEFINED) {
	    System.err.print(Position.line(position));
	    System.err.print(":");
	    System.err.print(Position.column(position));
	    System.err.print(": ");
	}
        System.err.println(message);
    }

    //########################################################################
    // Methods - file manipulation

    /** 
     * Open for reading the file passed as an argument. Return
     * the standard input if the argument is null or "-".
     */
    public InputStream openInput(String filename) {
        if (filename == null || filename.equals("-")) return System.in;
        try {
            return new FileInputStream(filename);
        } catch (FileNotFoundException exception) {
            throw fatal(Position.UNDEFINED, "cannot open file: " + filename);
        } catch (IOException exception) {
            throw fatal(Position.UNDEFINED, exception);
        }
    }

    /** 
     * Open for writing the file passed as an argument. Return
     * the standard ouput if the argument is null or "-".
     */
    public OutputStream openOutput(String filename) {
        if (filename == null || filename.equals("-")) return System.out;
        try {
            return new FileOutputStream(filename);
        } catch (FileNotFoundException exception) {
            throw fatal(Position.UNDEFINED, "cannot open file: " + filename);
        } catch (IOException exception) {
            throw fatal(Position.UNDEFINED, exception);
        }
    }

    /** Close the input stream. */
    public void close(InputStream in) {
        try {
            in.close();
        } catch (IOException exception) {
            fatal(Position.UNDEFINED, exception);
        }
    }

    /** Close the output stream. */
    public void close(OutputStream in) {
        try {
            in.close();
        } catch (IOException exception) {
            fatal(Position.UNDEFINED, exception);
        }
    }

    //########################################################################
}
